#include "gui.h"
#include "circles.xbm"

GtkWidget *text;
GtkWidget *window;
GdkColormap *colormap;

int
init_gui (argc, argv)
  int *argc;
  char ***argv;
{
	GtkWidget *hbox, *vbox;
	GtkWidget *scrollbar_v, *scrollbar_h;
	GdkBitmap *circles;

	gtk_init(argc, argv);

	colormap = gdk_colormap_get_system();

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window), "Xwatch");
	gtk_container_border_width(GTK_CONTAINER(window),0 );
	gtk_widget_set_usize(window, 130, 50);
	gtk_widget_realize(window);

	circles = gdk_bitmap_create_from_data (window->window, circles_bits, circles_width, circles_height);
	gdk_window_set_icon (window->window, NULL, circles, circles);
	//gdk_window_set_icon_name (window->window, "whatever");

	gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(destroy_window), NULL);
	gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(delete_window), NULL);

	vbox = gtk_hbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(window), vbox);
	hbox = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

	/* create text widget */
	text = gtk_text_new(NULL, NULL);
	gtk_text_set_editable(GTK_TEXT(text), FALSE);
	gtk_text_set_word_wrap(GTK_TEXT(text), TRUE);
	GTK_WIDGET_UNSET_FLAGS(text, GTK_CAN_FOCUS);
	gtk_box_pack_start(GTK_BOX(hbox), text, TRUE, TRUE, 0);
	gtk_widget_realize(text);
	gtk_text_set_point(GTK_TEXT(text), 0);
	gtk_widget_show(text);

	/* create horizontal scrollbar widget */
	if (make_scrollbar_h) {
		scrollbar_h = gtk_hscrollbar_new(GTK_TEXT(text)->hadj);
		GTK_WIDGET_UNSET_FLAGS(scrollbar_h, GTK_CAN_FOCUS);
		gtk_box_pack_start(GTK_BOX(hbox), scrollbar_h, FALSE, FALSE, 0);
		gtk_widget_show(scrollbar_h);
	}

	gtk_widget_show(hbox);

	/* create vertical scrollbar widget */
	if (make_scrollbar_v) {
		scrollbar_v = gtk_vscrollbar_new(GTK_TEXT(text)->vadj);
		gtk_box_pack_start(GTK_BOX(vbox), scrollbar_v, FALSE, FALSE, 0);
		GTK_WIDGET_UNSET_FLAGS(scrollbar_v, GTK_CAN_FOCUS);
		gtk_widget_show(scrollbar_v);
	}

	gtk_widget_show(vbox);
	
	return TRUE;
}

int
start_gui(void)
{
	gtk_widget_show(window);
	gtk_main();
	return TRUE;
}

void
exit_gui (code)
  int code;
{
	gtk_exit(code);
}

void input_event (data, source, condition)
  gpointer data;
  gint source;
  GdkInputCondition condition;
{
	if (condition == GDK_INPUT_READ)
		update_watch(WATCH(data));
}

gint
input_timeout (data)
  gpointer data;
{
	update_watch(WATCH(data));
	return TRUE;
}

gint
monitor_watch_event (watch)
  struct Watch *watch;
{
	return gdk_input_add(watch->fd, GDK_INPUT_READ, input_event, watch);
}

gint
monitor_watch_timed (watch)
  struct Watch *watch;
{
	return gtk_timeout_add(watch->delay, input_timeout, watch);
}

void
unmonitor_watch_event (watch)
  struct Watch *watch;
{
#ifdef __hpux
	gdk_input_remove((guint) watch->id); /* Its void - nothing to return */
#else
	return gdk_input_remove((guint) watch->id);
#endif
}

void
unmonitor_watch_timed (watch)
  struct Watch *watch;
{
#ifdef __hpux
	gtk_timeout_remove((guint) watch->id);
#else
	return gtk_timeout_remove((guint) watch->id);
#endif
}

void
print_line(watch, str, len, font, fgcolor, bgcolor)
  struct Watch *watch;
  char *str;
  int len;
  void *font;
  void *fgcolor;
  void *bgcolor;
{
	GdkColor *f=fgcolor, *b =bgcolor;
	if (f && b)
		printf("ins: fg=%d,%d,%d bg=%d,%d,%d\n",f->red,f->green,f->blue,b->red,b->green,b->blue);

	//gtk_text_freeze(GTK_TEXT(text));
	gtk_text_insert(GTK_TEXT(text), font, fgcolor, bgcolor, str, len);
	//gtk_text_thaw(GTK_TEXT(text));
}

gint
delete_window (widget, event, data)
  GtkWidget *widget;
  GdkEvent *event;
  gpointer data;
{
	return FALSE;
}

void
destroy_window (widget, data)
  GtkWidget *widget;
  gpointer data;
{
	close_watches();
	gtk_main_quit();
}


GdkFont *
load_font (str)
  char *str;
{
	return gdk_font_load(str);
}

void
unload_font (font)
  GdkFont *font;
{
	/* no operation */
}

#define ishexchar(a) ( ((a) >= '0' && (a) <= '9') || ((a) >= 'A' && (a) <= 'F') )
#define hexchar_to_int(a) ( ((a)>'9') ? ((a)-'A'+10) : ((a)-'0') )

/* load_color
 *   ...does manual parsing of a six digit hexadecimal string describing the color,
 *   to avoid the sscanf call.
 */
GdkColor *
load_color (str)
  char *str;
{
	GdkColor *color;
	int c;

#ifdef LOAD_COLOR_USE_SSCANF_PARSING
	color = malloc(sizeof(GdkColor));
	sscanf(str,"%2hx%2hx%2hx", &(color->red), &(color->green), &(color->blue));
#else
	if (strlen(str) < 6) return NULL;
	for (c=0; c < 6; c++) {
		if (!ishexchar(str[c])) return NULL;
	}
	color = malloc(sizeof(GdkColor));
	color->red = hexchar_to_int(str[0])*16 + hexchar_to_int(str[1]);
	color->green = hexchar_to_int(str[2])*16 + hexchar_to_int(str[3]);
	color->blue = hexchar_to_int(str[4])*16 + hexchar_to_int(str[5]);
#endif
	//printf("load_color: r=%x g=%x b=%x\n", color->red, color->green, color->blue);

	if (!gdk_color_alloc(colormap, color))
		return NULL;

	return color;
}

void
unload_color (color)
  GdkColor *color;
{
	free(color);
}
