#include "xwatch.h"

/* get_whole_line:
 *   returns a pointer to string in argument, if there is a full line in the
 *   argument (terminated by LF). the argument is updated to point to after
 *   the line returned. if there isn't a full line, NULL is returned.
 */
char *
get_whole_line(inputline)
  char **inputline;
{
	int c;
	char *outputline = *inputline;

	for(c = 0; (*inputline)[c] && (*inputline)[c] != '\n'; c++);
	if ((*inputline)[c] == '\n') {
		*inputline = &(*inputline)[c+1];
		return outputline;
	}

	return NULL;
}

/* print_pending_and_line:
 *   prints pending (if any) and first line in argument "line".
 */
void
print_pending_and_line(watch, line)
  struct Watch *watch;
  char *line;
{
	int len;

	/* we can assume that LF comes before null here, because we called
	 * get_whole_line first
	 */
	for (len=0; line[len] != '\n'; len++);
	len++;

	if (watch->pending) {
		watch->pending = realloc(watch->pending, watch->pendinglen+len);
		memcpy(&(watch->pending)[watch->pendinglen], line, len);
		output_line(watch, watch->pending, len+watch->pendinglen);
		free(watch->pending);
		watch->pending = NULL;
		watch->pendinglen = 0;
	} else {
		output_line(watch, line, len);
	}
}

/* append_to_pending:
 *   append data to pending line.
 */
void
append_to_pending(watch, line)
  struct Watch *watch;
  char *line;
{
	if (line[0]) {
		int len = strlen(line);

		if (watch->pending) {
			watch->pending = realloc(watch->pending, watch->pendinglen+len+1);
			memcpy(&(watch->pending)[watch->pendinglen], line, len);
			watch->pendinglen += len;
		} else {
			watch->pending = malloc(len);
			memcpy(watch->pending, line, len);
			watch->pendinglen = len;
		}
	} else {
		watch->pending = NULL;
	}
}
