#ifndef __XWATCH_XWATCH_H__
#define __XWATCH_XWATCH_H__

#ifdef HAVE_CONFIG_H
#  include "../config.h"
#endif

#ifndef _GNU_SOURCE
#  define _GNU_SOURCE
#endif

#if HAVE_UNISTD_H
#  include <sys/types.h>
#  include <unistd.h>
#else
#  ifndef STDIN_FILENO			/* shameless unistd.h definitions */
#    define STDIN_FILENO 0
#  endif
#  ifndef STDOUT_FILENO
#    define STDOUT_FILENO 1
#  endif
#  ifndef STDERR_FILENO
#    define STDERR_FILENO 2
#  endif
#endif

#if HAVE_FCNTL_H
#  include <fcntl.h>
#endif

#if TIME_WITH_SYS_TIME
#  include <sys/time.h>
#  include <time.h>
#else
#  if HAVE_SYS_TIME_H
#    include <sys/time.h>
#  else
#    include <time.h>
#  endif
#endif

#if HAVE_SYS_STAT_H
#  include <sys/stat.h>
#endif

#if STDC_HEADERS
#  include <stdlib.h>
#  include <stdarg.h>
#  include <string.h>
#  include <stdio.h>
#  include <ctype.h>
#  include <errno.h>
#  include <signal.h>
#else
#  if HAVE_VARARGS_H
#    include <varargs.h>
#  endif
#  if !HAVE_MEMCPY
#    define memcpy(d,s,n) bcopy((s), (d), (n))
#  endif
#  if !HAVE_STRDUP
char *strdup(char *);
#  endif
#endif

#include "common.h"

struct Watch {
	struct Watch *next;
	unsigned int type;			/* type of watch */
	int fd;						/* file descriptor */
	unsigned int delay;			/* delay between reads */
	unsigned int flags;			/* see below */
	void *font;
	void *fgcolor;
	void *bgcolor;
	void *id;					/* handle for input event/timeout */
	char *pending;				/* pending string (to be outputted) */
	unsigned int pendinglen;	/* length of pending string */
	void *ts_fgcolor;			/* timestamp foreground color */
	void *ts_bgcolor;			/* timestamp background color */
	void *ts_font;				/* timestamp font */
	char *timestamp;			/* timestamp string */
	char *filename;
};
#define WATCH(x) ((struct Watch *) x)

// watch flags
#define WATCH_FLAG_TIMESTAMP	1
#define WATCH_FLAG_SYSLOG		2

// watch types
#define WATCH_REGULAR 1
#define WATCH_FIFO 2
#define WATCH_SOCKET 3

#define WATCH_TYPE_PERIODICLY 1
#define WATCH_TYPE_EVENT 2

// program options
#define XWATCH_OPT_FILE 0
#define XWATCH_OPT_CONFIG 1
#define XWATCH_OPT_DELAY 2
#define XWATCH_OPT_FONT 3
#define XWATCH_OPT_FGCOLOR 4
#define XWATCH_OPT_BGCOLOR 5
#define XWATCH_OPT_TIMESTAMP 6
#define XWATCH_OPT_GEOMETRY 7

/* util.c */
int xread(int, void *, size_t);
void fatal(char *);
void *xmalloc(int);
void syslog_time_to_tm (char *, struct tm *);
void printerror(char *, ...);

/* string.c */
char *get_whole_line(char **);
void print_pending_and_line(struct Watch *, char *);
void append_to_pending(struct Watch *, char *);
void print_pending_and_line(struct Watch *, char *);

/* xwatch.c */
int open_file_watch(struct Watch *, char *);
void close_watches(void);
int parse_args(int, char **, char *);
void output_line (struct Watch *, char *, int);
void update_watch (struct Watch *);
void close_watch (struct Watch *);

extern int make_scrollbar_h;
extern int make_scrollbar_v;
extern int winx, winy, winh, winw;

#endif /* __XWATCH_XWATCH_H__ */
